/*
 "The contents of this file are subject to the Mozilla Public License
 
 Version 1.1 (the "License"); you may not use this file except in
 
 compliance with the License. You may obtain a copy of the License at
 
 http://www.mozilla.org/MPL/
 
 
 Software distributed under the License is distributed on an "AS IS"
 
 basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 
 License for the specific language governing rights and limitations
 
 under the License.
 
 
 The Original Code is OpenMCDF 1.5.4.
 
 
 The Initial Developer of the Original Code is Federico Blaseotto.
 
 Portions created by Federico Blaseotto are Copyright (C)2010 ~ 2013 Federico Blaseotto.
 
 All Rights Reserved.
 
 
 Contributor(s): Autodesk, Inc.."
 */

//
//  FDirectoryEntry.h
//  LIBCDF
//
//  Created by Weihua Luo on 4/3/2013.
//
#pragma once
#include "IFs/IFObject.h"
#include "IFs/IFComparable.h"
#include <string>
#include <ctype.h>
#include <algorithm>
#include "FileStream.h"
#include "FException.h"
#include "RWStream.h"
#include "LibCDF.h"
#include <vector>

typedef enum _tagStgType
{
    eStgInvalid = 0,
    eStgStorage = 1,
    eStgStream = 2,
    eStgLockbytes = 3,
    eStgProperty = 4,
    eStgRoot = 5
}StgType;

typedef enum _tagStgColor
{
    eRed = 0,
    eBlack = 1
}StgColor;

class LIBCDF_EXPORT IDirectoryEntry : public IFComparable, public RefObject
{
public:
	V_PROPERTY(int, SID);
    virtual const wchar_t* GetEntryName() const = 0;
    virtual void SetEntryName(const std::wstring& entryName) = 0;
	V_PROPERTY(unsigned short, NameLength);//2

	V_PROPERTY(StgColor, RBColor);//1
	V_PROPERTY(StgType, DEType);//1

	V_PROPERTY(int, Child);//4
	V_PROPERTY(int, LeftSibling);//4
	V_PROPERTY(int, RightSibling);//4

	V_PROPERTY(BYTE8, CreationDate);//8
	V_PROPERTY(BYTE8, ModifyDate);//8

	V_PROPERTY(int, StateBits);//4 User Tag
	V_PROPERTY(BYTE16, StorageCLSID);//16

	V_PROPERTY(int, StartSector);//4
	V_PROPERTY(DLONG, Size);//4 + 4:Not Used?

	virtual void Read(CFileStream* stream) = 0;
    virtual void Write(CFileStream* stream) = 0;
};

#define NOSTREAM (-1)
#define INT32 int
#define THIS_IS_GREATER 1
#define OTHER_IS_GREATER (-1)

class CFDirectoryEntry : public IDirectoryEntry
{
private:
	int sid;// = -1;
    std::wstring entryName;
    StgType stgType;
    StgColor stgColor;
    INT32 leftSibling;
    INT32 rightSibling;
    INT32 child;
    BYTE16 storageCLSID;
    INT32 stateBits;
    BYTE8 creationDate;
    BYTE8 modifyDate;
    INT32 startSector;
    DLONG size;
	CFDirectoryEntry(void);
public:
	CFDirectoryEntry(StgType argStgType);
	~CFDirectoryEntry(void);

	IMP_PROPERTY(int, SID, sid);

	const wchar_t* GetEntryName() const;

    void SetEntryName(const std::wstring& enName);

	virtual unsigned short NameLength() const {return entryName.length();};
	virtual void NameLength(const unsigned short& val) {throw NotImplementedException();}

	IMP_PROPERTY(StgType, DEType, stgType);
	IMP_PROPERTY(StgColor, RBColor, stgColor);

	IMP_PROPERTY(INT32, LeftSibling, leftSibling);
	IMP_PROPERTY(INT32, RightSibling, rightSibling);
	IMP_PROPERTY(INT32, Child, child);

	IMP_PROPERTY(BYTE16, StorageCLSID, storageCLSID);

	IMP_PROPERTY(INT32, StateBits, stateBits);

	IMP_PROPERTY(BYTE8, CreationDate, creationDate);
	IMP_PROPERTY(BYTE8, ModifyDate, modifyDate);

	IMP_PROPERTY(INT32, StartSector, startSector);
	IMP_PROPERTY(DLONG, Size, size);

    virtual int CompareTo(const IFComparable* obj) const;

    virtual bool Equals(const IFObject* obj) const;

	virtual void Delete();

private:
    /// <summary>
    /// FNV hash, short for Fowler/Noll/Vo
    /// </summary>
    /// <param name="buffer"></param>
    /// <returns>(not warranted) unique hash for byte array</returns>
	static unsigned int fnv_hash(unsigned char buffer[], int length);

public:
	int GetHashCode();
    
    void Write(CFileStream* stream);

    void Read(CFileStream* stream);
};

typedef std::vector<IDirectoryEntry*> CFDirectoryEntries;

