/*
 "The contents of this file are subject to the Mozilla Public License
 
 Version 1.1 (the "License"); you may not use this file except in
 
 compliance with the License. You may obtain a copy of the License at
 
 http://www.mozilla.org/MPL/
 
 
 Software distributed under the License is distributed on an "AS IS"
 
 basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 
 License for the specific language governing rights and limitations
 
 under the License.
 
 
 The Original Code is OpenMCDF 1.5.4.
 
 
 The Initial Developer of the Original Code is Federico Blaseotto.
 
 Portions created by Federico Blaseotto are Copyright (C)2010 ~ 2013 Federico Blaseotto.
 
 All Rights Reserved.
 
 
 Contributor(s): Autodesk, Inc.."
*/


//
//  FException.h
//  LIBCDF
//
//  Created by Weihua Luo on 4/3/2013.
//

#pragma once
#include <exception>
#include <string>
#include "IFs/IFObject.h"

namespace std
{
	class LIBCDF_EXPORT exception;
};

class LIBCDF_EXPORT CFException : public std::exception
{
public:
	CFException(void);
	CFException(const char* message);
	CFException(const std::string& message);
	virtual ~CFException(void);

public:
	virtual const char * what() const throw();
    
private:
    void CopyMessage(const char* message);
    char* m_pMessage;
};

/// <summary>
/// Raised when a data setter/getter method is invoked
/// on a stream or storage object after the disposal of the owner
/// compound file object.
/// </summary>

class LIBCDF_EXPORT CFDisposedException : public CFException
{
public:
	CFDisposedException()
        : CFException()
    {
    }


    CFDisposedException(std::string message)
        : CFException(message)
    {

    }
};

/// <summary>
/// Raised when opening a file with invalid header
/// or not supported COM/OLE Structured storage version.
/// </summary>

class LIBCDF_EXPORT CFFileFormatException : public CFException
{
public:
	CFFileFormatException()
        : CFException()
    {
    }

    CFFileFormatException(std::string message)
        : CFException(message)
    {

    }
};

/// <summary>
/// Raised when a named stream or a storage object
/// are not found in a parent storage.
/// </summary>

class LIBCDF_EXPORT CFItemNotFound : public CFException
{

public:
	CFItemNotFound()
        : CFException("Entry not found")
    {
    }

    CFItemNotFound(std::string message)
        : CFException(message)
    {

    }
};

/// <summary>
/// Raised when a method call is invalid for the current object state
/// </summary>

class LIBCDF_EXPORT CFInvalidOperation : public CFException
{
public:
	CFInvalidOperation()
        : CFException()
    {
    }


    CFInvalidOperation(std::string message)
        : CFException(message)
    {

    }
};

class LIBCDF_EXPORT CFArgumentOutOfRangeException : public CFException
{
public:
	CFArgumentOutOfRangeException()
        : CFException()
    {
    }


    CFArgumentOutOfRangeException(std::string message)
        : CFException(message)
    {

    }
};

class LIBCDF_EXPORT NotImplementedException : public CFException
{
public:
	NotImplementedException()
        : CFException()
    {
    }


    NotImplementedException(std::string message)
        : CFException(message)
    {

    }
};

/// <summary>
/// Raised when trying to add a duplicated CFItem
/// </summary>
/// <remarks>
/// Items are compared by name as indicated by specs.
/// Two items with the same name CANNOT be added within 
/// the same storage or sub-storage. 
/// </remarks>

class LIBCDF_EXPORT CFDuplicatedItemException : public CFException
{
public:
	CFDuplicatedItemException()
        : CFException()
    {
    }

	CFDuplicatedItemException(std::string message)
        : CFException(message)
    {

    }

};

/// <summary>
/// Raised when trying to load a Compound File with invalid, corrupted or mismatched fields (4.1 - specifications) 
/// </summary>
/// <remarks>
/// This exception is NOT raised when Compound file has been opened with NO_VALIDATION_EXCEPTION option.
/// </remarks>

class LIBCDF_EXPORT CFCorruptedFileException : public CFException
{
public:
	CFCorruptedFileException()
        : CFException()
    {
    }

    CFCorruptedFileException(std::string message)
        : CFException(message)
    {

    }

};
