/*
 "The contents of this file are subject to the Mozilla Public License
 
 Version 1.1 (the "License"); you may not use this file except in
 
 compliance with the License. You may obtain a copy of the License at
 
 http://www.mozilla.org/MPL/
 
 
 Software distributed under the License is distributed on an "AS IS"
 
 basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 
 License for the specific language governing rights and limitations
 
 under the License.
 
 
 The Original Code is OpenMCDF 1.5.4.
 
 
 The Initial Developer of the Original Code is Federico Blaseotto.
 
 Portions created by Federico Blaseotto are Copyright (C)2010 ~ 2013 Federico Blaseotto.
 
 All Rights Reserved.
 
 
 Contributor(s): Autodesk, Inc.."
*/


//
//  IFComparable.h
//  LIBCDF
//
//  Created by Weihua Luo on 4/3/2013.
//

#pragma once
#include "IFObject.h"

class LIBCDF_EXPORT IFComparable
{
public:
	virtual int CompareTo(const IFComparable* obj) const = 0;
};

template<class _T>
class ITComparer
{
public:
	virtual int Compare(_T* x, _T* y) = 0;
};

//_T should be the sub class of IFObject, IFComparable(optional)
template<class _T>
class TObjectComparer : public ITComparer<_T>
{
public:
	virtual int Compare(_T* x, _T* y)
	{
		if(!x || !y) return -1;
		IFComparable* plComp = dynamic_cast<IFComparable*>(x);
		IFComparable* prComp = dynamic_cast<IFComparable*>(y);
		if(plComp && prComp) return plComp->CompareTo(prComp);
		else
		{
			std::string slComp = x->ToString();
			std::string srComp = y->ToString();
			return strcmp(slComp.c_str(), srComp.c_str());
		}
	};
};
