/*
 "The contents of this file are subject to the Mozilla Public License
 
 Version 1.1 (the "License"); you may not use this file except in
 
 compliance with the License. You may obtain a copy of the License at
 
 http://www.mozilla.org/MPL/
 
 
 Software distributed under the License is distributed on an "AS IS"
 
 basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 
 License for the specific language governing rights and limitations
 
 under the License.
 
 
 The Original Code is OpenMCDF 1.5.4.
 
 
 The Initial Developer of the Original Code is Federico Blaseotto.
 
 Portions created by Federico Blaseotto are Copyright (C)2010 ~ 2013 Federico Blaseotto.
 
 All Rights Reserved.
 
 
 Contributor(s): Autodesk, Inc.."
*/

//
//  IFObject.h
//  LIBCDF
//
//  Created by Weihua Luo on 4/3/2013.
//

#pragma once

#include <string>

#ifdef __GNUC__
    #ifdef LIBCDF_EXPORTS
    #define LIBCDF_EXPORT __attribute__ ((visilibity ("default")))
    #else
    #define LIBCDF_EXPORT
    #endif
#else
    #ifdef LIBCDF_EXPORTS
    #define LIBCDF_EXPORT __declspec(dllexport)
    #else
    #define LIBCDF_EXPORT __declspec(dllimport)
    #endif
#endif

class LIBCDF_EXPORT IFObject
{
public:
        virtual bool Equals(const IFObject* obj) const = 0;

		static bool Equals(const IFObject* objA, const IFObject* objB){if(!objA) return false; return objA->Equals(objB);};

		virtual int GetHashCode() {return 0;};

		virtual const char* GetType() {return "IFObject";};

		virtual std::string ToString() const {return std::string("");};
		virtual std::wstring ToWString() const {return std::wstring(L"");};

		virtual void AddRef() = 0;
		virtual void ReleaseRef() = 0;
		virtual void Delete() = 0;
};

class LIBCDF_EXPORT RefObject : public IFObject
{
private:
	int _refCount;

public:
	RefObject() {_refCount = 0;}

	virtual void AddRef() { _refCount++; };
	virtual void ReleaseRef()
	{
		_refCount--;
		if (_refCount<=0) Delete();
	};
    
    int GetRefCount() const { return _refCount; }
};

// Summary:
//     Defines a method to release allocated resources.
class LIBCDF_EXPORT IFDisposable
{
    // Summary:
    //     Performs application-defined tasks associated with freeing, releasing, or
    //     resetting unmanaged resources.
    virtual void Dispose() = 0;
};

#define V_PROPERTY(type, pname) virtual type pname() const = 0;\
	virtual void pname(const type& val) = 0

#define IMP_PROPERTY(type, pname, var) virtual type pname() const {return var;};\
	virtual void pname(const type& val) {var = val;}

#define PROPERTY(type, pname, var) \
	void pname(const type& val) {var = val;} \
	type pname() const {return var;};
	
#define PROPERTY_P(ptype, pname, var) \
	void pname(ptype val) {var = val;} \
	ptype pname() const {return var;};
	
#define PROPERTY_GET(ptype, pname, var) \
	ptype pname() const {return var;};
	
