/*
 "The contents of this file are subject to the Mozilla Public License
 
 Version 1.1 (the "License"); you may not use this file except in
 
 compliance with the License. You may obtain a copy of the License at
 
 http://www.mozilla.org/MPL/
 
 
 Software distributed under the License is distributed on an "AS IS"
 
 basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 
 License for the specific language governing rights and limitations
 
 under the License.
 
 
 The Original Code is OpenMCDF 1.5.4.
 
 
 The Initial Developer of the Original Code is Federico Blaseotto.
 
 Portions created by Federico Blaseotto are Copyright (C)2010 ~ 2013 Federico Blaseotto.
 
 All Rights Reserved.
 
 
 Contributor(s): Autodesk, Inc.."
*/


#pragma once
#include "../IFs/IFComparable.h"
#include "RBTException.h"
#include "RBTNode.h"
#include <stack>


class RBTEnumerator : public RefObject
{
public:
	RBTEnumerator();
	RBTEnumerator(RBTNode* tnode, bool keys, bool ascending);

	virtual ~RBTEnumerator(){};

private:
	std::stack<RBTNode*> _stack;
	bool _keys;

	// return in ascending order (true) or descending (false)
	bool _ascending;
		
	// key
	IFComparable* _ordKey;
	// the data or value associated with the key
	IFObject* _objValue;

public:
	std::string  _Color;         // testing only, don't use in live system
    IFComparable* _parentKey;     // testing only, don't use in live system
		
	IFComparable* GetKey() const {return _ordKey;}
	void SetKey(IFComparable* pKey) {_ordKey = pKey;}

	IFObject* GetValue() const {return _objValue;}
	void SetValue(IFObject* pVal) {_objValue = pVal;}

	///<summary>
	/// HasMoreElements
	///</summary>
	bool HasMoreElements()
	{
		return (_stack.size() > 0);
	}
	///<summary>
	/// NextElement
	///</summary>
	IFObject* NextElement();
	bool MoveNext();

	virtual bool Equals(const IFObject *) const;

	virtual void Delete() {delete this;};

};