/*
 "The contents of this file are subject to the Mozilla Public License
 
 Version 1.1 (the "License"); you may not use this file except in
 
 compliance with the License. You may obtain a copy of the License at
 
 http://www.mozilla.org/MPL/
 
 
 Software distributed under the License is distributed on an "AS IS"
 
 basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 
 License for the specific language governing rights and limitations
 
 under the License.
 
 
 The Original Code is OpenMCDF 1.5.4.
 
 
 The Initial Developer of the Original Code is Federico Blaseotto.
 
 Portions created by Federico Blaseotto are Copyright (C)2010 ~ 2013 Federico Blaseotto.
 
 All Rights Reserved.
 
 
 Contributor(s): Autodesk, Inc.."
*/


//
//  FStreamView.cpp
//  LIBCDF
//
//  Created by Weihua Luo on 4/3/2013.
//
#include "StdAfx.h"
#include "FStreamView.h"
#include "FSector.h"

CFStreamView::CFStreamView(void)
{
	_position = 0;
	_sectorChain = NULL;
}


CFStreamView::~CFStreamView(void)
{
    //  don't clear _sectorChain, because we don't create any such
	//if (_sectorChain)
	//{
	//	_sectorChain->clear();
	//	delete _sectorChain;
	//}
}

void CFStreamView::AdjustLength(DLONG value, SectorQueue* availableSectors)
{
    _length = value;

    long delta = (long)(value - ((long)_sectorChain->size() * (long)_sectorSize));

    if (delta > 0)
    {
        // enlargment required

        int nSec = (int)ceil(((double)delta / _sectorSize));

        while (nSec > 0)
        {
            CFSectorPtr se;

			if (availableSectors == NULL || availableSectors->size() == 0)
            {
                se = new CFSector(_sectorSize, _stream);

            }
            else
            {
				se = availableSectors->front();
				availableSectors->pop();
            }

			_sectorChain->push_back(se);
            nSec--;
        }

    }
    else
    {
        // TODO: Freeing sector to avoid wasting space.

        // FREE Sectors
        //delta = Math.Abs(delta);
        //int nSec = (int)(_length - delta)  / _sectorSize;

        //if (((int)(_length - delta) % _sectorSize) != 0)
        //{
        //    nSec++;
        //}

        //while (_sectorChain->size() > nSec)
        //{
        //    _sectorChain.RemoveAt(_sectorChain->size() - 1);
		//	  // need to set -1 to the FAT...
        //}
    }
}

void CFStreamView::WriteByte(unsigned char value)
{
	memset(_buf.byte, 0, 4);
	_buf.byte[0] = value;
	Write((const unsigned char*)(_buf.byte), 0, 1);
}

void CFStreamView::Write(const CFByteArray& buffer, int offset, int count)
{
	Write(buffer.CByte(), offset, count);
}

void CFStreamView::Write(const unsigned char* buffer, int offset, int count)
{
    int byteWritten = 0;
    int roundByteWritten = 0;

    // Assure _length
    if ((_position + count) > _length)
        AdjustLength((_position + count));

    if (_sectorChain != NULL)
    {
        // First sector
        int secOffset = (int)(_position / (long)_sectorSize);
        int secShift = (int)_position % _sectorSize;

        roundByteWritten = (int)_MIN(_sectorSize - (int)(_position % (long)_sectorSize), count);

        if (secOffset < _sectorChain->size())
        {
            CFBuffer::BlockCopy(
                buffer,
                offset,
                (*_sectorChain)[secOffset]->GetData(),
                secShift,
                roundByteWritten
                );

            (*_sectorChain)[secOffset]->DirtyFlag(true);
        }

        byteWritten += roundByteWritten;
        offset += roundByteWritten;
        secOffset++;

        // Central sectors
        while (byteWritten < (count - _sectorSize))
        {
            roundByteWritten = _sectorSize;

            CFBuffer::BlockCopy(
                buffer,
                offset,
                (*_sectorChain)[secOffset]->GetData(),
                0,
                roundByteWritten
                );

            (*_sectorChain)[secOffset]->DirtyFlag(true);

            byteWritten += roundByteWritten;
            offset += roundByteWritten;
            secOffset++;
        }

        // Last sector
        roundByteWritten = count - byteWritten;

        if (roundByteWritten != 0)
        {
            CFBuffer::BlockCopy(
                buffer,
                offset,
                (*_sectorChain)[secOffset]->GetData(),
                0,
                roundByteWritten
                );

            (*_sectorChain)[secOffset]->DirtyFlag(true);

            offset += roundByteWritten;
            byteWritten += roundByteWritten;
        }

        _position += count;

    }
}
